/*----------------------------------------------------------------------------
 * Name:    main_s.c
 * Purpose: Main function secure mode
 *----------------------------------------------------------------------------*/

#include <arm_cmse.h>
#include "CMSDK_ARMv8MBL.h"                      /* Device header */
#include "Board_LED.h"                           /* ::Board Support:LED */

/* typedef for NonSecure callback functions */
typedef int32_t (*NonSecure_funcptr)(uint32_t) __attribute__((cmse_nonsecure_call));


/*----------------------------------------------------------------------------
  NonSecure callback functions
 *----------------------------------------------------------------------------*/
extern NonSecure_funcptr pfNonSecure_LED_On;
       NonSecure_funcptr pfNonSecure_LED_On  = (NonSecure_funcptr)NULL;
extern NonSecure_funcptr pfNonSecure_LED_Off;
       NonSecure_funcptr pfNonSecure_LED_Off = (NonSecure_funcptr)NULL;


/*----------------------------------------------------------------------------
  Secure functions exported to NonSecure application
 *----------------------------------------------------------------------------*/
int32_t Secure_LED_On (uint32_t num) __attribute__((cmse_nonsecure_entry));
int32_t Secure_LED_On (uint32_t num)
{
  return LED_On(num);
}

int32_t Secure_LED_Off (uint32_t num) __attribute__((cmse_nonsecure_entry)) ;
int32_t Secure_LED_Off (uint32_t num)
{
  return LED_Off(num);
}


/*----------------------------------------------------------------------------
  Secure function for NonSecure callbacks exported to NonSecure application
 *----------------------------------------------------------------------------*/
int32_t Secure_LED_On_callback(NonSecure_funcptr *callback) __attribute__((cmse_nonsecure_entry));
int32_t Secure_LED_On_callback(NonSecure_funcptr *callback)
{
  pfNonSecure_LED_On = (NonSecure_funcptr)cmse_nsfptr_create(callback);
	return 0;
}

int32_t Secure_LED_Off_callback(NonSecure_funcptr *callback) __attribute__((cmse_nonsecure_entry));
int32_t Secure_LED_Off_callback(NonSecure_funcptr *callback)
{
  pfNonSecure_LED_Off = (NonSecure_funcptr)cmse_nsfptr_create(callback);
	return 0;
}


/*----------------------------------------------------------------------------
  SysTick IRQ Handler
 *----------------------------------------------------------------------------*/
void SysTick_Handler (void);
void SysTick_Handler (void) {
  static uint32_t ticks;

  switch (ticks++) {
    case   0:
      LED_On (0u);
      break;
    case 200:
      LED_Off(0u);
      break;
		case 300:
      if (pfNonSecure_LED_On != NULL)
      {
        pfNonSecure_LED_On(1u);
      }
      break;
		case 500:
      if (pfNonSecure_LED_Off != NULL)
      {
        pfNonSecure_LED_Off(1u);
      }
      break;
		case 600:
      ticks = 0;
      break;

    default:
      if (ticks > 600) {
        ticks = 0;
      }
  }
}


void NonSecure_Start (uint32_t addr) __attribute__((always_inline));
void NonSecure_Start (uint32_t addr)
{
  __ASM volatile ("MOV r0, %0\n\t"
                  "BLXNS   r0\n\t" : : "r" (addr));
}


/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
#if defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
__ASM(" .global __ARM_use_no_argv\n");
#endif
int main (void)
{
  volatile uint32_t x;
  volatile uint32_t NonSecure_ResetHandler;

  /* exercise some core register from Secure Mode */
  x = __get_MSP();
  x = __get_PSP();
  __TZ_set_MSP_NS(0x21000000u);
  x = __TZ_get_MSP_NS();
  __TZ_set_PSP_NS(0x22000000u);
  x = __TZ_get_PSP_NS();

  LED_Initialize ();

  SystemCoreClockUpdate();
  SysTick_Config(SystemCoreClock/100);  /* Generate interrupt each 10 ms */

  NonSecure_ResetHandler = cmse_nsfptr_create(*((uint32_t *)(0x00200000u + 4u)));
  NonSecure_Start(NonSecure_ResetHandler);
}

